@@ -12,11 +12,73 @@
 
 #include "common/e-m365-connection.h"
 #include "common/e-m365-tz-utils.h"
+#include "e-cal-backend-m365-recur-blob.h"
 #include "e-cal-backend-m365-utils.h"
 
 #define EC_ERROR_EX(_code, _msg) e_client_error_create (_code, _msg)
 #define ECC_ERROR_EX(_code, _msg) e_cal_client_error_create (_code, _msg)
 
+static void
+ecb_m365_json_to_ical_recur_blob (JsonObject *m365_object,
+				  ETimezoneCache *timezone_cache,
+				  ICalComponent **inout_icomp)
+{
+	const gchar *base64_blob;
+	ICalTimezone *recur_zone = NULL;
+	ICalProperty *prop;
+	GSList *extra_detached = NULL;
+
+	g_return_if_fail (m365_object != NULL);
+	g_return_if_fail (inout_icomp != NULL);
+	g_return_if_fail (*inout_icomp != NULL);
+
+	if (!e_cal_util_component_has_recurrences (*inout_icomp))
+		return;
+
+	base64_blob = e_m365_json_get_string_single_value_extended_property (m365_object, E_M365_RECURRENCE_BLOB_NAME);
+	if (!base64_blob || !*base64_blob)
+		return;
+
+	prop = i_cal_component_get_first_property (*inout_icomp, I_CAL_DTSTART_PROPERTY);
+	if (prop) {
+		ICalParameter *param;
+
+		param = i_cal_property_get_first_parameter (prop, I_CAL_TZID_PARAMETER);
+		if (param) {
+			const gchar *tzid;
+
+			tzid = i_cal_parameter_get_tzid (param);
+
+			if (tzid && *tzid)
+				recur_zone = e_timezone_cache_get_timezone (timezone_cache, tzid);
+
+			g_clear_object (&param);
+		}
+
+		g_clear_object (&prop);
+	}
+
+	if (e_cal_backend_m365_decode_recur_blob (base64_blob, *inout_icomp, recur_zone, &extra_detached)) {
+		if (extra_detached) {
+			ICalComponent *vcalendar;
+			GSList *link;
+
+			vcalendar = i_cal_component_new_vcalendar ();
+			i_cal_component_take_component (vcalendar, *inout_icomp);
+
+			for (link = extra_detached; link; link = g_slist_next (link)) {
+				ICalComponent *icomp = link->data;
+
+				i_cal_component_take_component (vcalendar, icomp);
+			}
+
+			*inout_icomp = vcalendar;
+
+			g_slist_free (extra_detached);
+		}
+	}
+}
+
 static void
 ecb_m365_get_uid (EM365Connection *cnc,
 		  const gchar *group_id,
@@ -1648,7 +1710,6 @@ ecb_m365_add_recurrence (EM365Connection *cnc,
 	}
 
 	if (i_cal_component_count_properties (new_comp, I_CAL_RDATE_PROPERTY) > 0 ||
-	    i_cal_component_count_properties (new_comp, I_CAL_EXDATE_PROPERTY) > 0 ||
 	    i_cal_component_count_properties (new_comp, I_CAL_EXRULE_PROPERTY) > 0) {
 		g_propagate_error (error, EC_ERROR_EX (E_CLIENT_ERROR_NOT_SUPPORTED,
 			_("Microsoft 365 calendar cannot store component with RDATE, EXDATE or RRULE properties")));
@@ -2079,6 +2140,38 @@ ecb_m365_add_online_meeting (EM365Connection *cnc,
 	return TRUE;
 }
 
+static gchar *
+ecb_m365_calc_hash (const gchar *value)
+{
+	gchar *hash;
+
+	if (!value)
+		return g_strdup ("empty");
+
+	/* the MD5 is fine here, the id-s are not that long to cause the collision;
+	   it's also short, which is for good */
+	hash = g_compute_checksum_for_string (G_CHECKSUM_MD5, value, -1);
+	if (!hash)
+		hash = g_compute_checksum_for_string (G_CHECKSUM_SHA1, value, -1);
+	if (!hash)
+		hash = g_compute_checksum_for_string (G_CHECKSUM_SHA256, value, -1);
+
+	/* fallback when cannot compute the hash */
+	if (!hash) {
+		gchar *ptr;
+
+		hash = g_strdup (value);
+
+		/* remove forward- and back-slashes from the string */
+		for (ptr = hash; *ptr; ptr++) {
+			if (*ptr == '/' || *ptr == '\\')
+				*ptr = '_';
+		}
+	}
+
+	return hash;
+}
+
 static gboolean
 ecb_m365_get_attachments (EM365Connection *cnc,
 			  const gchar *group_id,
@@ -2093,6 +2186,7 @@ ecb_m365_get_attachments (EM365Connection *cnc,
 {
 	GSList *attachments = NULL, *link;
 	const gchar *id;
+	gchar *event_id_hash;
 	gboolean success = TRUE;
 
 	switch (i_cal_component_isa (inout_comp)) {
@@ -2103,7 +2197,7 @@ ecb_m365_get_attachments (EM365Connection *cnc,
 		id = e_m365_event_get_id (m365_object);
 
 		if (!e_m365_connection_list_event_attachments_sync (cnc, NULL,
-			group_id, folder_id, id, "id,name,contentType,contentBytes",
+			group_id, folder_id, id, "id,name,contentType",
 			&attachments, cancellable, error)) {
 			return FALSE;
 		}
@@ -2115,40 +2209,31 @@ ecb_m365_get_attachments (EM365Connection *cnc,
 		return FALSE;
 	}
 
+	event_id_hash = ecb_m365_calc_hash (id);
+
 	for (link = attachments; link && success; link = g_slist_next (link)) {
 		CamelStream *content_stream;
 		EM365Attachment *m365_attach = link->data;
-		gchar *filename;
+		gchar *filename, *attach_id_hash;
 
 		if (!m365_attach || e_m365_attachment_get_data_type (m365_attach) != E_M365_ATTACHMENT_DATA_TYPE_FILE ||
 		    !e_m365_attachment_get_name (m365_attach))
 			continue;
 
-		filename = g_build_filename (attachments_dir, id, e_m365_attachment_get_id (m365_attach), NULL);
+		attach_id_hash = ecb_m365_calc_hash (e_m365_attachment_get_id (m365_attach));
+
+		filename = g_build_filename (attachments_dir, event_id_hash, NULL);
+		g_mkdir_with_parents (filename, 0777);
+		g_free (filename);
+
+		filename = g_build_filename (attachments_dir, event_id_hash, attach_id_hash, NULL);
 
 		content_stream = camel_stream_fs_new_with_name (filename, O_CREAT | O_TRUNC | O_WRONLY, 0666, error);
 
 		if (content_stream) {
-			CamelMimeFilter *filter;
-			CamelStream *filter_stream;
-			const gchar *base64_data;
-
-			filter_stream = camel_stream_filter_new (content_stream);
-
-			filter = camel_mime_filter_basic_new (CAMEL_MIME_FILTER_BASIC_BASE64_DEC);
-			camel_stream_filter_add (CAMEL_STREAM_FILTER (filter_stream), filter);
-			g_object_unref (filter);
-
-			base64_data = e_m365_file_attachment_get_content_bytes (m365_attach);
-
-			if (base64_data && *base64_data)
-				success = camel_stream_write (filter_stream, base64_data, strlen (base64_data), cancellable, error) != -1;
-
-			camel_stream_flush (filter_stream, cancellable, NULL);
-			g_object_unref (filter_stream);
-
-			camel_stream_flush (content_stream, cancellable, NULL);
-			g_object_unref (content_stream);
+			success = e_m365_connection_get_event_attachment_sync (cnc, NULL, group_id, folder_id, id,
+				e_m365_attachment_get_id (m365_attach), e_m365_connection_util_read_raw_data_cb, content_stream,
+				cancellable, error);
 
 			if (success) {
 				gchar *uri;
@@ -2199,9 +2284,11 @@ ecb_m365_get_attachments (EM365Connection *cnc,
 		}
 
 		g_free (filename);
+		g_free (attach_id_hash);
 	}
 
 	g_slist_free_full (attachments, (GDestroyNotify) json_object_unref);
+	g_free (event_id_hash);
 
 	return success;
 }
@@ -2716,6 +2803,9 @@ e_cal_backend_m365_utils_json_to_ical (EM365Connection *cnc,
 		}
 	}
 
+	if (success && kind == I_CAL_VEVENT_COMPONENT)
+		ecb_m365_json_to_ical_recur_blob (m365_object, timezone_cache, &icomp);
+
 	if (!success)
 		g_clear_object (&icomp);
 
